<?php


namespace FiloBlu\ExtInventory\Model;

use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Exception\CouldNotSaveException;
use FiloBlu\ExtInventory\Model\ResourceModel\InventoryFbOrderitemSource as ResourceInventoryFbOrderitemSource;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceInterfaceFactory;
use FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceSearchResultsInterfaceFactory;
use FiloBlu\ExtInventory\Model\ResourceModel\InventoryFbOrderitemSource\CollectionFactory as InventoryFbOrderitemSourceCollectionFactory;
use Magento\Framework\Exception\NoSuchEntityException;
use FiloBlu\ExtInventory\Api\InventoryFbOrderitemSourceRepositoryInterface;

class InventoryFbOrderitemSourceRepository implements InventoryFbOrderitemSourceRepositoryInterface
{

    protected $dataObjectHelper;

    private $storeManager;

    protected $inventoryFbOrderitemSourceFactory;

    protected $searchResultsFactory;

    protected $dataObjectProcessor;

    protected $extensionAttributesJoinProcessor;

    private $collectionProcessor;

    protected $extensibleDataObjectConverter;
    protected $resource;

    protected $inventoryFbOrderitemSourceCollectionFactory;

    protected $dataInventoryFbOrderitemSourceFactory;


    /**
     * @param ResourceInventoryFbOrderitemSource $resource
     * @param InventoryFbOrderitemSourceFactory $inventoryFbOrderitemSourceFactory
     * @param InventoryFbOrderitemSourceInterfaceFactory $dataInventoryFbOrderitemSourceFactory
     * @param InventoryFbOrderitemSourceCollectionFactory $inventoryFbOrderitemSourceCollectionFactory
     * @param InventoryFbOrderitemSourceSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     * @param ExtensibleDataObjectConverter $extensibleDataObjectConverter
     */
    public function __construct(
        ResourceInventoryFbOrderitemSource $resource,
        InventoryFbOrderitemSourceFactory $inventoryFbOrderitemSourceFactory,
        InventoryFbOrderitemSourceInterfaceFactory $dataInventoryFbOrderitemSourceFactory,
        InventoryFbOrderitemSourceCollectionFactory $inventoryFbOrderitemSourceCollectionFactory,
        InventoryFbOrderitemSourceSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource = $resource;
        $this->inventoryFbOrderitemSourceFactory = $inventoryFbOrderitemSourceFactory;
        $this->inventoryFbOrderitemSourceCollectionFactory = $inventoryFbOrderitemSourceCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataInventoryFbOrderitemSourceFactory = $dataInventoryFbOrderitemSourceFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
        $this->collectionProcessor = $collectionProcessor;
        $this->extensionAttributesJoinProcessor = $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        \FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceInterface $inventoryFbOrderitemSource
    ) {
        /* if (empty($inventoryFbOrderitemSource->getStoreId())) {
            $storeId = $this->storeManager->getStore()->getId();
            $inventoryFbOrderitemSource->setStoreId($storeId);
        } */
        
        $inventoryFbOrderitemSourceData = $this->extensibleDataObjectConverter->toNestedArray(
            $inventoryFbOrderitemSource,
            [],
            \FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceInterface::class
        );
        
        $inventoryFbOrderitemSourceModel = $this->inventoryFbOrderitemSourceFactory->create()->setData($inventoryFbOrderitemSourceData);
        
        try {
            $this->resource->save($inventoryFbOrderitemSourceModel);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the inventoryFbOrderitemSource: %1',
                $exception->getMessage()
            ));
        }
        return $inventoryFbOrderitemSourceModel->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function get($inventoryFbOrderitemSourceId)
    {
        $inventoryFbOrderitemSource = $this->inventoryFbOrderitemSourceFactory->create();
        $this->resource->load($inventoryFbOrderitemSource, $inventoryFbOrderitemSourceId);
        if (!$inventoryFbOrderitemSource->getId()) {
            throw new NoSuchEntityException(__('inventory_fb_orderitem_source with id "%1" does not exist.', $inventoryFbOrderitemSourceId));
        }
        return $inventoryFbOrderitemSource->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->inventoryFbOrderitemSourceCollectionFactory->create();
        
        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceInterface::class
        );
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        \FiloBlu\ExtInventory\Api\Data\InventoryFbOrderitemSourceInterface $inventoryFbOrderitemSource
    ) {
        try {
            $inventoryFbOrderitemSourceModel = $this->inventoryFbOrderitemSourceFactory->create();
            $this->resource->load($inventoryFbOrderitemSourceModel, $inventoryFbOrderitemSource->getInventoryFbOrderitemSourceId());
            $this->resource->delete($inventoryFbOrderitemSourceModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the inventory_fb_orderitem_source: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($inventoryFbOrderitemSourceId)
    {
        return $this->delete($this->get($inventoryFbOrderitemSourceId));
    }
}
